// directly, apart from some name changes and removals, ripped off from NeHe tutorial #30
// (read explanation near function do2005Physics() in beginning.cc)
#ifndef tvector_h
#define tvector_h

#include <math.h>
template <class T> inline T sqr(T const & m) {
	return m * m;
}

class Ray;

class Vector {
	public:
		enum Status { INVALID, DEFAULT, UNIT };

	private:
		double _x, _y, _z;
		Status _Status;

		// Constructors
		Vector(double x, double y, double z, Status s) : _x(x), _y(y), _z(z), _Status(s) {}

	public:
		// Constructors
		Vector() : _x(0.0), _y(0.0), _z(0.0), _Status(INVALID) {}
		Vector(double x, double y, double z) : _x(x), _y(y), _z(z), _Status(DEFAULT) {}

		// Mid point between two lines
		Vector(const Ray &line1, const Ray &line2);

		// Selectors
		double X() const { return _x; }
		double Y() const { return _y; }
		double Z() const { return _z; }
		int isUnit() const { return _Status==UNIT; }
		int isDefault() const { return _Status==DEFAULT; }
		int isValid() const { return _Status!=INVALID; }

		// Change the status of a vector
		Vector &unit();
		static Vector &unit(const Vector &v, Vector &result) { result = v; return result.unit(); }
		static Vector unit(const Vector &v) { return Vector(v).unit(); }

		Vector &Default();
		static Vector Default(const Vector &v, Vector &result) { result = v; return result.Default(); }
		static Vector Default(const Vector &v) { return Vector(v).Default(); }

		// Magnitude
		double mag() const { return (isValid() ? (isUnit() ? 1.0 : sqrt(sqr(X()) + sqr(Y()) + sqr(Z()))) : 0.0); }
		double magSqr() const { return (isValid() ? (isUnit() ? 1.0 : sqr(X()) + sqr(Y()) + sqr(Z())) : 0.0); }

		// Dot or scalar product
		double dot(const Vector &v) const { return ((isValid() && v.isValid()) ? (X()*v.X() + Y()*v.Y() + Z()*v.Z()) : 0.0); }
		static double dot(const Vector &v1, const Vector &v2) { return v1.dot(v2); }

		// Distance between two vectors
		double dist(const Vector &v) const { return (*this-v).mag(); }
		double distSqr(const Vector &v) const { return (*this-v).magSqr(); }

		// Optimised arithmetic methods
		static Vector &add(const Vector &v1, const Vector &v2, Vector &result);
		static Vector &subtract(const Vector &v1, const Vector &v2, Vector &result);
		static Vector &cross(const Vector &v1, const Vector &v2, Vector &result);
//		static Vector &invert(const Vector &v1, Vector &result);
		static Vector &invert(const Vector &v1, Vector result);
		static Vector &multiply(const Vector &v1, const double &scale, Vector &result);

		// Vector arithmetic, addition, subtraction and vector product
		Vector operator-() const { return invert(*this, Vector()); }
		Vector &operator+=(const Vector &v) { return add(*this, v, *this); }
		Vector &operator-=(const Vector &v) { return subtract(*this, v, *this); }
		Vector &operator*=(const Vector &v) { Vector tv(*this); return cross(tv, v, *this); }
		Vector &operator*=(const double &scale) { return multiply(*this, scale, *this); }
		Vector operator+(const Vector &v) const { Vector tv; return add(*this, v, tv); }
		Vector operator-(const Vector &v) const { Vector tv; return subtract(*this, v, tv); }
		Vector operator*(const Vector &v) const { Vector tv; return cross(*this, v, tv); }
		Vector operator*(const double &scale) const { Vector tv; return multiply(*this, scale, tv); }
};
#endif
