///////////////////////////////////////////////////////////////////////////////
// This file is part of "MMV - MegaloManic Venture", which is a demo which took
// part in Ohjelmointiputka's "Demokilpailu 2005" in January 2005.
//
// Copyleft Matti "Deewiant" Niemenmaa, 2004-2005.
//
// Filename: general.cc
//           Handy functions which are used throughout the program.
//
// File created: 2005-01-02
///////////////////////////////////////////////////////////////////////////////

#include "header.hh"
#include <cstring>

/******************************************************************************\
Function pushScreenCoordinateMatrix()

Makes OpenGL coordinates identical to window coordinates (pixels).
\******************************************************************************/
void pushScreenCoordinateMatrix()
{
	glPushAttrib(GL_TRANSFORM_BIT);
	int viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0[viewport], viewport[2], 1[viewport], viewport[3]);
	glPopAttrib();
	return;
}

/******************************************************************************\
Function popProjectionMatrix()

Pops the projection matrix without changing the matrix mode.
\******************************************************************************/
void popProjectionMatrix()
{
	glPushAttrib(GL_TRANSFORM_BIT);
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glPopAttrib();
	return;
}

unsigned short int const daysInMonth[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

/******************************************************************************\
Function counterGoto()

Increases the date counter to the wanted date. Basic functionality is thus:
If target month is not current month, add days until end of month reached and
increase month, repeat; else add days until target day reached.

Inputs:
	targMonth, targDay     The wanted date: month and day, respectively.
\******************************************************************************/
void counterGoto(short targMonth, short targDay)
{
	// pass it as a "real" month, but in-program they go from 0 to 11...
	--targMonth;

	bool done = false, grabTime = true;
	double aika = 0;

	do {
		if (grabTime) {
			aika = glfwGetTime();
			grabTime = false;
		}

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		doLower();
		kirjoita(fonts[proFontTiny], 32, 188, "Time passes.\nIt is getting dark.\nYou are likely to be eaten by a grue.");
		setTop();
		doEarth();

		// around 14 days per second
		if ((glfwGetTime() - aika) >= 0.07) {
			if (month != targMonth) {
				if (day != daysInMonth[month])
					++day;
				else { ++month; day = 1; }
			} else if (day != targDay)
				++day;
			else done = true;
			grabTime |= true;
		}

		glfwSwapBuffers();

	} while (!done && glfwGetWindowParam(GLFW_OPENED));

	return;
}

/******************************************************************************\
Function doPhysics()

Shows the "World Year of Physics" text in the corner and handles the bouncing
balls.
\******************************************************************************/
void doPhysics()
{
	pushScreenCoordinateMatrix();
	GLUquadricObj * quadric = gluNewQuadric();

	for (int i = 0; i < 10; ++i) {
		glColor3ub(192, 192, 192);
		glPushMatrix();
		glColor3ub(192, 192, 192);
		glTranslated(bouncyBalls[i].x, bouncyBalls[i].y, 3);
		gluSphere(quadric, 4, 32, 32);
		glColor3ub(192, 192, 192);
		glPopMatrix();
		glColor3ub(192, 192, 192);

		// bounce off letters
		if (bouncyBalls[i].y <= 80)
			bouncyBalls[i].yVel = 5;
		else // gravity
			bouncyBalls[i].yVel -= 0.1;

		bouncyBalls[i].y += bouncyBalls[i].yVel;
	}

	glColor3f(1, 1, 1);
	kirjoita(fonts[proFontTiny], 888, 64, "World Year\n of Physics");

	popProjectionMatrix();

	return;
}

/******************************************************************************\
Function doCounter()

Shows the current day, month, and year.
\******************************************************************************/
void doCounter()
{
	kirjoita(fonts[proFontBig], 376, 32, "%u", year);

	if (counter) {
		char buffer[10];
		switch (month) {
			case  0: std::strcpy(buffer, "January"  ); break;
			case  1: std::strcpy(buffer, "February" ); break;
			case  2: std::strcpy(buffer, "March"    ); break;
			case  3: std::strcpy(buffer, "April"    ); break;
			case  4: std::strcpy(buffer, "May"      ); break;
			case  5: std::strcpy(buffer, "June"     ); break;
			case  6: std::strcpy(buffer, "July"     ); break;
			case  7: std::strcpy(buffer, "August"   ); break;
			case  8: std::strcpy(buffer, "September"); break;
			case  9: std::strcpy(buffer, "October"  ); break;
			case 10: std::strcpy(buffer, "November" ); break;
			case 11: std::strcpy(buffer, "December" ); break;
		}
		kirjoita(fonts[proFont], 380, 144, "%s %u", buffer, day);
	}

	return;
}

/******************************************************************************\
Function doLower()

Sets the viewport to the "status bar" at the bottom, and calls doPhysics() and
doCounter().
\******************************************************************************/
void doLower()
{
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glViewport(0, 0, 1024, 256);
	glLoadIdentity();

	doPhysics();
	doCounter();

	return;
}

/******************************************************************************\
Function setTop()

Sets the viewport to the main area, above the "status bar".
\******************************************************************************/
void setTop()
{
	glViewport(0, 256, 1024, 512);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50, 2, 0, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// from now on:
	// Y-coordinates in the top frame are half a unit per pixel
	// so it starts from 256 units, ends at 512 units, but it's 512 pixels tall
	// very strange...
	// distorts the Earth and makes my work generally harder - circles are not circles, have to halve their y-coordinates first...
	// but I can't figure out why this happens so I just have to leave it... it only makes stuff a lot harder, not impossible
	// and it seems harder to fix the bug than to just take it into account... bad programming practice, but hey, there's a deadline...
	// X-coordinates work normally - 1024 pixels, 1024 units

	return;
}

/******************************************************************************\
Function NormalVector()

This is one of those functions whose creators I forgot to take note of.
Apparently this computes the normal to 3 vectors.

Inputs:
	p1, p2, p3    The three vectors.
	n             Where to place the result.
\******************************************************************************/
void NormalVector(GLdouble p1[3], GLdouble p2[3], GLdouble p3[3], GLdouble n[3])
{
	GLdouble v1[3], v2[3], d;
	// calculate two vectors, using the middle point
	// as the common origin
	v1[0] = p3[0] - p1[0];
	v1[1] = p3[1] - p1[1];
	v1[2] = p3[2] - p1[2];
	v2[0] = p3[0] - p2[0];
	v2[1] = p3[1] - p2[1];
	v2[2] = p3[2] - p2[2];

	// calculate the cross product of the two vectors
	n[0] = v1[1] * v2[2] - v2[1] * v1[2];
	n[1] = v1[2] * v2[0] - v2[2] * v1[0];
	n[2] = v1[0] * v2[1] - v2[0] * v1[1];

	// normalize the vector
	d = ( n[0] * n[0] + n[1] * n[1] + n[2] * n[2] );
	// try to catch very small vectors
	if (d < static_cast<GLdouble>(0.00000001)) {
		d = static_cast<GLdouble>(100000000.0);
	} else {
		d = static_cast<GLdouble>(1.0) / std::sqrt(d);
	}

	n[0] *= d;
	n[1] *= d;
	n[2] *= d;

	return;
}

bool increaseX = true, increaseZ = true, takeTime = true;
double valoAika;

/******************************************************************************\
Function doEarth()

Draws the Earth. Also handles its lighting and rotation.
\******************************************************************************/
void doEarth()
{
	if (!drawEarth)
		return;

	glPushMatrix();

	glRotated(rotation[0], 1, 0, 0);
	glRotated(rotation[1], 0, 1, 0);

	if (takeTime) {
		valoAika = glfwGetTime();
		takeTime = false;
	}

	// spin the light
	// the 4 points we want to cycle through are:
	// x  0, z -1 (start)
	// x  1, z  0
	// x  0, z  1
	// x -1, z  0
	// where x is [0] and z is [2]
	glLightfv(GL_LIGHT1, GL_POSITION, atmosLightPosition);
	glEnable(GL_LIGHT1);

	if      (atmosLightPosition[0] >=  0 && atmosLightPosition[2] <= -1) increaseZ = true;
	else if (atmosLightPosition[0] >=  1 && atmosLightPosition[2] >=  0) increaseX = false;
	else if (atmosLightPosition[0] <=  0 && atmosLightPosition[2] >=  1) increaseZ = false;
	else if (atmosLightPosition[0] <= -1 && atmosLightPosition[2] <=  0) increaseX = true;

	// update the lighting only tence per second
	if ((glfwGetTime() - valoAika) >= 0.1) {
		if (increaseX == true) atmosLightPosition[0] += 0.03;
		else                   atmosLightPosition[0] -= 0.03;
		if (increaseZ == true) atmosLightPosition[2] += 0.03;
		else                   atmosLightPosition[2] -= 0.03;
		takeTime = true;
	}

	glEnable(GL_LIGHTING);

	popProjectionMatrix();

	// draw the Earth
	int const NumLatitudes  = 36;
	int const NumLongitudes = 72;
	GLdouble u[3], v[3], w[3], n[3];
	double start_lat, start_lon, theta1, theta2, phi1, phi2, lat_incr, lon_incr;

	// I spent ~3 hours debugging Earth spinning which screwed up the textures...
	// Eventually, I realised depth testing wasn't on and I was seeing the
	// front and back of the Earth at the same time.
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textures[earth]);

	glBegin(GL_TRIANGLES);
	start_lat = -90;
	start_lon =   0;
	lat_incr = 5; // 180 / NumLatitudes
	lon_incr = 5; // 360 / NumLongitudes

	for (int col = 0; col < NumLongitudes; ++col) {
		phi1 = (start_lon +  col      * lon_incr) * GEO::DE2RA;
		phi2 = (start_lon + (col + 1) * lon_incr) * GEO::DE2RA;

		for (int row = 0; row < NumLatitudes; ++row) {
			theta1 = (start_lat +  row      * lat_incr) * GEO::DE2RA;
			theta2 = (start_lat + (row + 1) * lat_incr) * GEO::DE2RA;

			// why are some of these halved? see setTop()
			u[0] = 0.5 * std::cos(phi1) * std::cos(theta1);
			u[1] = 1.0 * std::sin(theta1);
			u[2] = 0.5 * std::sin(phi1) * std::cos(theta1);

			v[0] = 0.5 * std::cos(phi1) * std::cos(theta2);
			v[1] = 1.0 * std::sin(theta2);
			v[2] = 0.5 * std::sin(phi1) * std::cos(theta2);

			w[0] = 0.5 * std::cos(phi2) * std::cos(theta2);
			w[1] = 1.0 * std::sin(theta2);
			w[2] = 0.5 * std::sin(phi2) * std::cos(theta2);

			NormalVector(u, v, w, n);
			glNormal3dv(n);
			glTexCoord2d((180.0 - phi1 * GEO::RA2DE) / 360.0, (theta1 + GEO::PIOVER2) * GEO::RA2DE / 180.0);
			glVertex3dv(u);
			glTexCoord2d((180.0 - phi1 * GEO::RA2DE) / 360.0, (theta2 + GEO::PIOVER2) * GEO::RA2DE / 180.0);
			glVertex3dv(v);
			glTexCoord2d((180.0 - phi2 * GEO::RA2DE) / 360.0, (theta2 + GEO::PIOVER2) * GEO::RA2DE / 180.0);
			glVertex3dv(w);

			v[0] = 0.5 * std::cos(phi2) * std::cos(theta1);
			v[1] = 1.0 * std::sin(theta1);
			v[2] = 0.5 * std::sin(phi2) * std::cos(theta1);

			NormalVector(u, w, v, n);
			glNormal3dv(n);
			glTexCoord2d((180.0 - phi1 * GEO::RA2DE) / 360.0, (theta1 + GEO::PIOVER2) * GEO::RA2DE / 180.0);
			glVertex3dv(u);
			glTexCoord2d((180.0 - phi2 * GEO::RA2DE) / 360.0, (theta2 + GEO::PIOVER2) * GEO::RA2DE / 180.0);
			glVertex3dv(w);
			glTexCoord2d((180.0 - phi2 * GEO::RA2DE) / 360.0, (theta1 + GEO::PIOVER2) * GEO::RA2DE / 180.0);
			glVertex3dv(v);
		}
	}
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);

	pushScreenCoordinateMatrix();

	glPopMatrix();

	return;
}

