///////////////////////////////////////////////////////////////////////////////
// This file is part of "MMV - MegaloManic Venture", which is a demo which took
// part in Ohjelmointiputka's "Demokilpailu 2005" in January 2005.
//
// Copyleft Matti "Deewiant" Niemenmaa, 2004-2005.
//
// Filename: middle.cc
//           All the code for the run-through of the year 2005
//
// File created: 2005-01-02
///////////////////////////////////////////////////////////////////////////////

#include "header.hh"

// used only in Doom()
bool drawEarth = true;

/******************************************************************************\
Struct star

The information to represent a single "star" - a particle in a firework.
\******************************************************************************/
struct star {
	double x, y, xVel, yVel;
	unsigned char r, g, b;
};

/******************************************************************************\
Function newYearFireworks()

The fireworks on January 1, 2005.
\******************************************************************************/
void newYearFireworks()
{
	double newX, newY;
	month = 0, day = 1;
	double aika;
	bool uusi[11] = {true, false, false, false, false, false, false, false, false, false, false};
	bool   on[11] = {true, false, false, false, false, false, false, false, false, false, false};
	star stars[11][512];

	for (int j = 0; j < 11; ++j) {
		// why are newY 384 and not 512? see setTop()
		switch (j) {
			case  0: newX = 512; newY = 408; break;
			case  1: newX = 256; newY = 384; break;
			case  2: newX = 264; newY = 416; break;
			case  3: newX = 504; newY = 288; break;
			case  4: newX = 640; newY = 448; break;
			case  5: newX = 544; newY = 304; break;
			case  6: newX = 192; newY = 336; break;
			case  7: newX = 832; newY = 464; break;
			case  8: newX = 456; newY = 352; break;
			case  9: newX = 448; newY = 272; break;
			case 10: newX = 508; newY = 388; break;
		}
		for (int i = 0; i < 512; ++i) {
			stars[j][i].x = newX;
			stars[j][i].y = newY;
		}
	}



	// makes more sense to start fireworks at night
	atmosLightPosition[0] = 1;
	atmosLightPosition[2] = 1;

	srand(37);

	glfwSetTime(-1);

	do {
		aika = glfwGetTime();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		doLower();
		kirjoita(fonts[proFontTiny], 32, 188, "New Year is celebrated around\nthe world. Fireworks are fired aplenty,\nat least in Western Europe.");
		setTop();
		doEarth();

		glDisable(GL_LIGHTING);
		glPushMatrix();
		for (int j = 0; j < 11; ++j) {
			if (on[j]) {
				// if they haven't existed before, give them velocity
				if (uusi[j]) {
					for (int i = 0; i < 512; ++i) {
						do {
							stars[j][i].xVel = (rand()%2 ? 1 : -1) * (rand() / static_cast<float>(RAND_MAX) * 5);
							stars[j][i].yVel = (rand()%2 ? 1 : -1) * (rand() / static_cast<float>(RAND_MAX) * 5);
						} while ((stars[j][i].xVel * stars[j][i].xVel + stars[j][i].yVel * stars[j][i].yVel) > 25);

						stars[j][i].r = rand()%255;
						stars[j][i].g = rand()%255;
						stars[j][i].b = rand()%255;
					}
					// they definitely exist now...
					uusi[j] = false;
				}

				for (int i = 0; i < 512; ++i) {
					stars[j][i].x += stars[j][i].xVel;
					stars[j][i].y += stars[j][i].yVel;

					// if the stars are inside the viewport, draw them
					if (stars[j][i].x < 1024 && stars[j][i].x > 0 && stars[j][i].y > 256 && stars[j][i].y < 512) {
						glBegin(GL_LINE_STRIP);
							glColor3ub(0, 0, 0);
							glVertex3d( (stars[j][i].x - stars[j][i].xVel * 4), (stars[j][i].y - stars[j][i].yVel * 4), -1);
							glColor3ub(stars[j][i].r , stars[j][i].g , stars[j][i].b);
							glVertex3d(stars[j][i].x, stars[j][i].y, -1);
						glEnd();
					}
				}
			}
		}
		glPopMatrix();
		glEnable(GL_LIGHTING);

		if (aika >= 4.0 && aika < 4.03)
			uusi[10] = on[10] = true;
		else if (aika >= 3.49 && aika < 3.52)
			uusi[9] = on[9] = true;
		else if (aika >= 3.2 && aika < 3.23)
			uusi[8] = on[8] = true;
		else if (aika >= 3.0 && aika < 3.03)
			uusi[7] = on[7] = true;
		else if (aika >= 2.79 && aika < 2.82)
			uusi[6] = on[6] = true;
		else if (aika >= 2.49 && aika < 2.52)
			uusi[5] = on[5] = true;
		else if (aika >= 1.99 && aika < 2.02)
			uusi[4] = on[4] = true;
		else if (aika >= 1.74 && aika < 1.77)
			uusi[3] = on[3] = true;
		else if (aika >= 0.89 && aika < 0.92)
			uusi[2] = on[2] = true;
		else if (aika >= 0.39 && aika < 0.42)
			uusi[1] = on[1] = true;

		if (aika >= 7) {
			for (int j = 0; j < 11; ++j) {
				for (int i = 0; i < 512; ++i) {
					if (stars[j][i].r > 0) --stars[j][i].r;
					if (stars[j][i].g > 0) --stars[j][i].g;
					if (stars[j][i].b > 0) --stars[j][i].b;
				}
			}
		}

		glfwSwapBuffers();

	} while (aika < 10 && glfwGetWindowParam(GLFW_OPENED));

	return;
}

/******************************************************************************\
Function Ireland()

Ireland's internationalization regarding its speed limits.
\******************************************************************************/
void Ireland()
{
	double aika;
	takeTime = true;
	unsigned char brightSquare = 0, brightPoly = 0, brightText = 254, brightText2 = 255;
	int i = 0;

	glfwSetTime(0);

	do {
		aika = glfwGetTime();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		doLower();
		kirjoita(fonts[proFontTiny], 32, 188, "The Republic of Ireland\ninternationalises, in that it\nchanges all its speed limits and\nthe street signs depicting\nthem from \"mph\" to \"kph\".");
		setTop();
		doEarth();

		if (rotation[0] > -56)
			rotation[0] -= 0.5;
		else {
			glDisable(GL_LIGHTING);
			glPushMatrix();

			if (aika > 11) {
				glTranslated(i, 0, 0);
				i += 5;
			}

			if (brightSquare < 192) ++brightSquare;
			if (brightPoly   < 255) ++brightPoly;

			glColor3ub(brightSquare, brightSquare, brightSquare);
			glBegin(GL_QUADS);
				glVertex3d(504, 352, -1);
				glVertex3d(504, 416, -1);
				glVertex3d(512, 416, -1);
				glVertex3d(512, 352, -1);
			glEnd();
			glColor3ub(brightPoly, brightPoly, brightPoly);
			glBegin(GL_POLYGON);
				glVertex3d(496, 416, -1);
				glVertex3d(472, 428, -1);
				glVertex3d(472, 440, -1);
				glVertex3d(496, 452, -1);
				glVertex3d(520, 452, -1);
				glVertex3d(544, 440, -1);
				glVertex3d(544, 428, -1);
				glVertex3d(520, 416, -1);
			glEnd();

			if (brightPoly == 255) {
				if (aika <= 7 && brightText > 0) brightText -= 2;

				if (brightText != 254) {
					glColor3ub(brightText, brightText, brightText);
					kirjoita(fonts[proFontSmall], 496, 436, "65");
					kirjoita(fonts[proFontSmall], 490, 421, "mph");
				}

				if (aika > 7 && brightText < 254) {
					brightText += 2;
				} else if (aika > 7 && brightText == 254) {
					if (brightText2 > 0) --brightText2;
					glColor3ub(brightText2, brightText2, brightText2);
					kirjoita(fonts[proFontSmall], 488, 436, "100");
					kirjoita(fonts[proFontSmall], 490, 421, "kph");
				}
			}

			glPopMatrix();
			glEnable(GL_LIGHTING);
		}

		glfwSwapBuffers();

	} while (aika < 13 && glfwGetWindowParam(GLFW_OPENED));

	return;
}

/******************************************************************************\
Function chineseNewYear()

The fireworks at the time of the Chinese New Year.
\******************************************************************************/
void chineseNewYear()
{
	double aika, newX, newY;
	bool uusi [6] = {true, false, false, false, false, false};
	bool on   [6] = {true, false, false, false, false, false};
	bool timeSet = false;
	star stars[6][512];

	srand(41);

	glfwSetTime(0);

	for (int j = 0; j < 6; ++j) {
		// we want them to be in China, hence reset coordinates until they are
		do {
			newX =       (rand() / static_cast<float>(RAND_MAX)) * 1024;
			newY = 256 + (rand() / static_cast<float>(RAND_MAX)) *  256; // why not 512? See setTop()
		} while (!(newX >= 472 && newX <= 576 && newY >= 408 && newY <= 440));

		for (int i = 0; i < 512; ++i) {
			stars[j][i].x = newX;
			stars[j][i].y = newY;
		}
	}

	do {
		aika = glfwGetTime();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		doLower();
		kirjoita(fonts[proFontTiny], 32, 188, "New Year is celebrated in China.\nThe Year of the Rooster begins.\nHappiness ensues.");
		setTop();
		doEarth();

		if (rotation[1] < 190)
			rotation[1] += 0.5;
		else if (rotation[0] < -25)
			rotation[0] += 0.5;
		else {
			// zero the time first time through, so we don't fire fireworks while rotating
			if (!timeSet) {
				glfwSetTime(0);
				timeSet |= true;
			}

			glDisable(GL_LIGHTING);
			glPushMatrix();
			for (int j = 0; j < 6; ++j) {
				if (on[j]) {
					if (uusi[j]) {
						for (int i = 0; i < 512; ++i) {
							do {
								stars[j][i].xVel = (rand()%2 ? 1 : -1) * (rand() / static_cast<float>(RAND_MAX) * 5);
								stars[j][i].yVel = (rand()%2 ? 1 : -1) * (rand() / static_cast<float>(RAND_MAX) * 5);
							} while ((stars[j][i].xVel * stars[j][i].xVel + stars[j][i].yVel * stars[j][i].yVel) > 25);

							stars[j][i].r = rand()%255;
							stars[j][i].g = rand()%255;
							stars[j][i].b = rand()%255;
						}
						uusi[j] ^= true;
					}

					for (int i = 0; i < 512; ++i) {
						stars[j][i].x += stars[j][i].xVel;
						stars[j][i].y += stars[j][i].yVel;

						if (stars[j][i].x < 1024 && stars[j][i].x > 0 && stars[j][i].y > 256 && stars[j][i].y < 512) {
							glBegin(GL_LINE_STRIP);
								glColor3ub(0, 0, 0);
								glVertex3d( (stars[j][i].x - stars[j][i].xVel * 4), (stars[j][i].y - stars[j][i].yVel * 4), -1);
								glColor3ub(stars[j][i].r , stars[j][i].g , stars[j][i].b);
								glVertex3d(stars[j][i].x, stars[j][i].y, -1);
							glEnd();
						}
					}
				}
			}
			glPopMatrix();
			glEnable(GL_LIGHTING);

			if (aika >= 0.99 && aika < 1.02)
				uusi[5] = on[5] = true;
			else if (aika >= 0.89 && aika < 0.92)
				uusi[4] = on[4] = true;
			else if (aika >= 0.74 && aika < 0.77)
				uusi[3] = on[3] = true;
			else if (aika >= 0.39 && aika < 0.42)
				uusi[2] = on[2] = true;
			else if (aika >= 0.19 && aika < 0.22)
				uusi[1] = on[1] = true;

			if (aika >= 4) {
				for (int j = 0; j < 6; ++j) {
					for (int i = 0; i < 512; ++i) {
						if (stars[j][i].r > 0) --stars[j][i].r;
						if (stars[j][i].g > 0) --stars[j][i].g;
						if (stars[j][i].b > 0) --stars[j][i].b;
					}
				}
			}
		}

		glfwSwapBuffers();

	} while (aika < 7 && glfwGetWindowParam(GLFW_OPENED));

	return;
}

/******************************************************************************\
Function eclipse()

Handles both the eclipses - the hybrid one and the annular one. Note the 4 pixel
difference in radius!

Inputs:
	which   Whether it is the hybrid or annular eclipse we are dealing with.
\******************************************************************************/
void eclipse(int which)
{
	double aika;
	takeTime = true;
	double trans = 0, angle, s[4], m[4], xS, yS, xM, yM, path = 0;
	bool done = false;

	s[2] = xS = 512;
	s[3] = yS = 384;
	m[2] = xM = 1024;
	m[3] = yM = 0;

	glfwSetTime(-50);

	do {
		aika = glfwGetTime();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		doLower();
		kirjoita(fonts[proFontTiny], 32, 188, (which == 0 ? "An annular-total, or hybrid, solar\neclipse occurs. It is total above\nthe South Pacific ocean." : "An annular solar eclipse takes place,\ncentered around Spain."));
		setTop();

		if ((which == 0 ? (rotation[1] < 245) : (rotation[1] > 85))) {
			if (!which)
				rotation[1] += 0.5;
			else rotation[1] -= 0.5;
			doEarth();
		} else if ((which == 0 ? (rotation[0] > 15) : (rotation[0] < -50))) {
			if (!which)
				rotation[0] -= 0.5;
			else rotation[0] += 0.5;
			doEarth();
		} else {
			glPushMatrix();
			glTranslated(trans, 0, 0);
			if (trans < 1.6 && !done)
				trans += 0.0125;
			doEarth();
			glPopMatrix();

			if (trans >= 1.6) {
				glDisable(GL_LIGHTING);
				glPushMatrix();

				// sun
				glColor3f(1, 1, 0);
				glBegin(GL_TRIANGLES);
					for (int i = 0; i <= 360; ++i) {
						angle = i / GEO::RA2DE;

						s[0] = xS + 80 * std::sin(angle);
						s[1] = yS + 40 * std::cos(angle); // why halved? see setTop()

						glVertex3d(xS, yS, -1);
						glVertex3d(s[2], s[3], -1);
						glVertex3d(s[0], s[1], -1);

						s[2] = s[0];
						s[3] = s[1];
					}
				glEnd();

				// moon movement
				if (path < 1.56875)
					path += 0.00625;
				if (path >= 1.56875 && done == false) {
					// we only start caring about time after the eclipse takes place
					glfwSetTime(0);
					done = true;
				}
				if (path < 3.5 && done == true && aika >= 2)
					path += 0.00625;

				// moon
				glColor3ub(224, 224, 224);
				glBegin(GL_TRIANGLES);
					for (int i = 0; i <= 360; ++i) {
						angle = i / GEO::RA2DE;

						// the path along which the moon will move
						xM = 512 + 512 * std::cos(path);
						yM = 384 * std::sin(path); // why 384? see setTop()

						m[0] = xM + (which == 0 ? 80 : 76) * std::sin(angle);
						m[1] = yM + (which == 0 ? 40 : 38) * std::cos(angle); // why halved? see setTop()

						glVertex3d(xM, yM, -1);
						glVertex3d(m[2], m[3], -1);
						glVertex3d(m[0], m[1], -1);

						m[2] = m[0];
						m[3] = m[1];
					}
				glEnd();

				glPopMatrix();
				glEnable(GL_LIGHTING);
			}

			if (path >= 3.4 && trans > 0)
				trans -= 0.0125;
			if (done && trans <= 0)
				break;
		}

		glfwSwapBuffers();

	} while (aika < 15 && glfwGetWindowParam(GLFW_OPENED));

	return;
}

/******************************************************************************\
Function hhgttg()

The destruction of the Earth and its replacement with an intergalactic highway,
or whatever it was... don't have the book at hand.

Short for "HitchHiker's Guide To The Galaxy".
\******************************************************************************/
void hhgttg()
{
	double aika;
	takeTime |= true;

	int const numStars = 2048;
	double rot = 5;
	int yOffset = 384;

	star stars[numStars];

	srand(768);

	for (int i = 0; i < numStars; ++i) {
		stars[i].x = 496 + rand()%32;
		stars[i].y = 368 + rand()%32;
		stars[i].xVel = (rand()%2 ? 1 : -1) * (rand() / static_cast<float>(RAND_MAX) * 3);
		stars[i].yVel = (rand()%2 ? 1 : -1) * (rand() / static_cast<float>(RAND_MAX) * 3);
		do
			stars[i].r = rand()%255;
		while (stars[i].r < 64);
	}

	glfwSetTime(0);

	do {
		aika = glfwGetTime();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		doLower();
		kirjoita(fonts[proFontTiny], 32, 188, "The \"The Hitchhiker's Guide to the Galaxy\"\nmovie is released.\n\nConsider the scene above a trailer.");
		setTop();

		if (aika < 3)
			doEarth();
		else if (aika >= 3) {
			pushScreenCoordinateMatrix();

			for (int i = 0; i < numStars; ++i) {
				stars[i].x += stars[i].xVel;
				stars[i].y += stars[i].yVel;

				if (stars[i].x < 1024 && stars[i].x > 0 && stars[i].y > 256 && stars[i].y < 512) {
					glBegin(GL_LINE_STRIP);
						glColor3ub(0, 0, 0);
						glVertex3d( (stars[i].x - stars[i].xVel * 4), (stars[i].y - stars[i].yVel * 4), -1);
						glColor3ub(stars[i].r , 0, 0);
						glVertex3d(stars[i].x, stars[i].y, -1);
					glEnd();
				}
			}

			if (yOffset == 0 && aika > 12) {

				if (rotation[0]) rotation[0] = 0;
				if (rotation[1]) rotation[1] = 0;

				popProjectionMatrix();

				glTranslated(-rot, 0, 0);
				if (rot > 0)
					rot -= 0.04;
				doEarth();

			}

			if (aika > 6 && aika < 12) {

				// the highway:
				// light gray pavement, top and bottom
				// dark gray middle
				// yellow stripes

				// why are the y-values smaller than they should be? see setTop()
				glBegin(GL_QUADS);
					// top pavement
					glColor3ub(176, 176, 176);

					glVertex3d(0,    512 + yOffset, -1);
					glVertex3d(0,    480 + yOffset, -1);
					glVertex3d(1024, 480 + yOffset, -1);
					glVertex3d(1024, 512 + yOffset, -1);

					// highway itself
					glColor3ub(64, 64, 64);

					glVertex3d(0,    288 + yOffset, -1);
					glVertex3d(0,    480 + yOffset, -1);
					glVertex3d(1024, 480 + yOffset, -1);
					glVertex3d(1024, 288 + yOffset, -1);

					// bottom pavement
					glColor3ub(176, 176, 176);

					glVertex3d(0,    256 + yOffset, -1);
					glVertex3d(0,    288 + yOffset, -1);
					glVertex3d(1024, 288 + yOffset, -1);
					glVertex3d(1024, 256 + yOffset, -1);
				glEnd();
				glBegin(GL_QUADS);
					// stripes
					glColor3ub(250, 240, 100);

					glVertex2d( 64, 376 + yOffset);
					glVertex2d( 64, 392 + yOffset);
					glVertex2d(192, 392 + yOffset);
					glVertex2d(192, 376 + yOffset);

					glVertex2d(320, 376 + yOffset);
					glVertex2d(320, 392 + yOffset);
					glVertex2d(448, 392 + yOffset);
					glVertex2d(448, 376 + yOffset);

					glVertex2d(576, 376 + yOffset);
					glVertex2d(576, 392 + yOffset);
					glVertex2d(704, 392 + yOffset);
					glVertex2d(704, 376 + yOffset);

					glVertex2d(832, 376 + yOffset);
					glVertex2d(832, 392 + yOffset);
					glVertex2d(960, 392 + yOffset);
					glVertex2d(960, 376 + yOffset);
				glEnd();

				if (yOffset > 0)
					yOffset -= 2;

				day = 6;

			} else if (aika < 6) day = 42;

		}

		glfwSwapBuffers();

	} while (aika < 16 && glfwGetWindowParam(GLFW_OPENED));

	return;
}

/******************************************************************************\
Function NorwayIndependence()

The Norway flag.
\******************************************************************************/
void NorwayIndependence()
{
	double aika;
	takeTime = true;

	double vertex[45][45][3], temp;
	int rippleCounter = 0;

	for (int x = 0; x < 45; ++x) {
		// the 1/5 is the width/height of the flag
		// the 1/5 in the sine relates somehow to the smoothness of the ripple
		// don't mess with it is what I'd say, looks crap otherwise
		// the 40 relates to the ripple, hard to explain exactly how...
		for (int y = 0; y < 45; ++y) {
			vertex[x][y][0] = x / 5.0;
			vertex[x][y][1] = y / 5.0;
			vertex[x][y][2] = std::sin(x / 5.0 * 40.0 * GEO::DE2RA);
		}
	}

	glfwSetTime(0);

	do {
		aika = glfwGetTime();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		doLower();
		kirjoita(fonts[proFontTiny], 32, 188, "Norway celebrates her centenary of\nindependence.");
		setTop();

		if (rotation[1] < 92) {
			rotation[1] += 0.5;
			doEarth();
		} else if (rotation[0] > -70) {
			rotation[0] -= 0.5;
			doEarth();
			if (rotation[0] == -70)
				glfwSetTime(0);
		} else if (aika < 4 || aika > 11) {
			doEarth();
			if (aika < 3)
				kirjoita(fonts[proFontSmall], 64, 480, "What's that red thing there?");
			else if (aika < 4) kirjoita(fonts[proFontSmall], 64, 480, "Let's see...");
			glDisable(GL_LIGHTING);
			glBegin(GL_LINES);
				glColor3f(1, 0, 0);
				glVertex3d(528, 392, -1);
				glVertex3d(530, 392, -1);
			glEnd();
			glEnable(GL_LIGHTING);
		} else {
			pushScreenCoordinateMatrix();
			glBegin(GL_QUADS);
				glColor3ub(26, 42, 0);
				glVertex3d(0, 256, 1);
				glVertex3d(0, 512, 1);
				glVertex3d(1024, 512, 1);
				glVertex3d(1024, 0, 1);
			glEnd();
			popProjectionMatrix();
			glTranslated(-4, -4, -12);

			glColor3f(1, 1, 1);

			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, textures[norway]);
			glBegin(GL_QUADS);
			for (int x = 0; x < 44; ++x) {
				for (int y = 0; y < 44; ++y ) {

					double texX =   x    / 44.0;
					double texY =   y    / 44.0;
					double texX2 = (x+1) / 44.0;
					double texY2 = (y+1) / 44.0;

					glTexCoord2d(texX, texY);
					glVertex3d(vertex[x][y][0], vertex[x][y][1], vertex[x][y][2]);

					glTexCoord2d(texX, texY2);
					glVertex3d(vertex[x][y+1][0], vertex[x][y+1][1], vertex[x][y+1][2]);

					glTexCoord2d(texX2, texY2);
					glVertex3d(vertex[x+1][y+1][0], vertex[x+1][y+1][1], vertex[x+1][y+1][2]);

					glTexCoord2d(texX2, texY);
					glVertex3d(vertex[x+1][y][0], vertex[x+1][y][1], vertex[x+1][y][2]);
				}
			}
			glEnd();
			glDisable(GL_TEXTURE_2D);

			// every two frames, ripple
			if (rippleCounter == 2) {
				for (int y = 0; y < 45; ++y) {

					temp = vertex[0][y][2];

					for (int x = 0; x < 44; ++x)
						vertex[x][y][2] = vertex[x+1][y][2];

					vertex[44][y][2] = temp;
				}
				rippleCounter = 0;
			}
			++rippleCounter;
		}

		glfwSwapBuffers();

	} while (aika < 12 && glfwGetWindowParam(GLFW_OPENED));

	return;
}

/******************************************************************************\
Function Doom()

Doom's 12th birthday! Imps dancing around.

See note on Audiere here. Unfortunate, but I really didn't have time.
\******************************************************************************/
void Doom()
{
	double aika, firstTime = 500;
	takeTime = true;

	double xPos = -256, yPos = 384, xPos2, yPos2;
	bool firstDone = false, secondDone = false;

	glfwSetTime(0);

	// initialize Audiere... done here because I had very little time when I
	// started incorporating Audiere, and when I tried to play the sound
	// through a pointer I got myself a runtime error

	AudioDevicePtr device(OpenDevice());
  	if (!device) {
		std::fprintf(stderr, "Could not initialise Audiere.");
		std::exit(true);
	}
	OutputStreamPtr theSound(OpenSound(device, "Data/death.wav", false));
	if (!theSound) {
		std::fprintf(stderr, "Could not load the sound file Data/death.wav.");
		std::exit(true);
	}

	do {
		aika = glfwGetTime();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		doLower();
		kirjoita(fonts[proFontTiny], 32, 188, "Doom is 12. Jubilation ensues.");
		setTop();
		doEarth();

		pushScreenCoordinateMatrix();

		glDisable(GL_LIGHTING);
		glPushMatrix();

		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, textures[imp]);

		if (xPos < 64 && !firstDone) {
			xPos += 2;
			if (xPos == 64) {
				firstDone ^= true;
				firstTime = aika;
			}
		}

		if (firstDone && !secondDone) {
			if (aika - firstTime > 2 && xPos != 832)
				xPos -= 2;
			if (xPos < -144) {
				xPos = 832;
				yPos = 64;
			}
			if (yPos < 640 && xPos == 832)
				yPos += 2;
			if (yPos > 528)
				secondDone |= true;
		}

		if (secondDone) {
			if (xPos != 128) {
				xPos = 128;
				yPos = 256;
			}

			xPos2 = 1024 - 128;
			yPos2 = 512;

			glColor3d(1, 0, 0);

			if (drawEarth) {
				drawEarth &= false;
				theSound->play();
			}

		} else {
			xPos2 = xPos + 128;
			yPos2 = yPos + 64;
		}

		// yPos increase is lower because... see setTop()
		glBegin(GL_QUADS);
			glTexCoord2d(0, 0);
			glVertex3d(xPos,  yPos,  -1);
			glTexCoord2d(0, 1);
			glVertex3d(xPos,  yPos2, -1);
			glTexCoord2d(1, 1);
			glVertex3d(xPos2, yPos2, -1);
			glTexCoord2d(1, 0);
			glVertex3d(xPos2, yPos,  -1);
		glEnd();

		glPopMatrix();
		glEnable(GL_LIGHTING);

		glfwSwapBuffers();

	} while (aika < 14 && glfwGetWindowParam(GLFW_OPENED));

	return;
}

